<?php

namespace Drupal\datadog_metrics;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\Queue\QueueWorkerManagerInterface;
use Drupal\datadog_metrics\Utils\MetricTypes;
use Drupal\datadog_metrics\Utils\MonitoringUtils;
use GuzzleHttp\ClientInterface;

class QueueMonitoring {
  use MonitoringUtils;

  public QueueWorkerManagerInterface $workerManager;
  public QueueFactory $queueFactory;
  public ClientInterface $httpClient;
  public ConfigFactoryInterface $configFactory;
  public ImmutableConfig $datadogConfig;

  public function __construct(
    QueueWorkerManagerInterface $workerManager,
    QueueFactory $queueFactory,
    ClientInterface $httpClient,
    ConfigFactoryInterface $configFactory,
  ) {
    $this->workerManager = $workerManager;
    $this->queueFactory = $queueFactory;
    $this->httpClient = $httpClient;
    $this->configFactory = $configFactory;
    $this->datadogConfig = $configFactory->get('datadog.settings');
  }

  /**
   * Queue Monitoring
   *
   * @command queue-monitoring
   * @aliases qm
   * @usage queue-monitoring
   */
  public function queueMonitoring() {
    $queues = $this->getQueues();
    $monitoringInformation = $this->getMonitoringInformation($queues);
    $this->submitToDatadog($monitoringInformation, $this->datadogConfig, $this->httpClient);
  }

  public function getMonitoringInformation(array $queues): array {
    $monitoringInformation = [];

    foreach (array_keys($queues) as $queueId) {
      $queue = $this->queueFactory->get($queueId);

      $metricTags = [
        "host:" . $this->datadogConfig->get('env'),
        "queue:" . $queueId,
      ];

      $monitoringInformation[] =
        $this->getMonitoringInformationEntry('queues.items', MetricTypes::COUNT, $queue->numberOfItems(), $metricTags);

    }

    return $monitoringInformation;
  }

  public function getQueues(): array {
    $queues = [];
    foreach ($this->workerManager->getDefinitions() as $name => $definiton) {
      $queues[$name] = $definiton;
    }
    return $queues;
  }

}
